  /**
    * @file BrushlessAVR.c
    * @brief This module contains the implementation of the brushless hardware
    * functions.

    * @author Stefan Popovic
    *
    * @date 11/09/2016
    *
    * @version 0.1a
    *
    * @copyright GNU Public License
    *
    * Test configuration:
    *
    * MCU: ATMEGA32
    * Dev.Board: EasyAVR v7 Connectivity
    * External Clock 08.0000 MHz
    * Ext. Modules: Brushless Click Board
    * SW: MikroC PRO for AVR v6.1.2

    * NOTES:
      - Place Brushless click board on mikroBUS slot 1.
      - Make sure you turn ON the PORTD LEDs (SW10.7).
      - Put Button Press Level switch PB into upper position and pull-down PORTB pins 0 and 1.
      - PWM: PD4
      - Vsense: PA7
      - Int: PD2




    /**************************************************************************
    * Module Preprocessor Constants
    **************************************************************************/
    #define CW    0x01
    #define CCW   0x00
    #define STEP  1

    /**************************************************************************
    * Module Preprocessor Macros
    **************************************************************************/


    /**************************************************************************
    * Module Typedefs
    **************************************************************************/
    
     /**************************************************************************
     * Includes
     **************************************************************************/
     #include "built_in.h"

    /**************************************************************************
    * Module Variable Definitions
    **************************************************************************/
    sbit MOTOR_DIR at PORTA6_bit;

    // LCD module connections
    sbit LCD_RS at PORTA2_bit;
    sbit LCD_EN at PORTD6_bit;
    sbit LCD_D4 at PORTC4_bit;
    sbit LCD_D5 at PORTC5_bit;
    sbit LCD_D6 at PORTC6_bit;
    sbit LCD_D7 at PORTC7_bit;

    sbit LCD_RS_Direction at DDA2_bit;
    sbit LCD_EN_Direction at DDD6_bit;
    sbit LCD_D4_Direction at DDC4_bit;
    sbit LCD_D5_Direction at DDC5_bit;
    sbit LCD_D6_Direction at DDC6_bit;
    sbit LCD_D7_Direction at DDC7_bit;
    // End LCD module connections

    unsigned int current_speed;
    unsigned int pwm_period;
    char txt[24];
    char txt1[] = "PWM Duty";
    char txt2[] = "PWM MAX";

    void MCU_Init()
    {
      ADC_Init(); // ADC Init
      pwm_period = 256;
      PWM16bit_Init(_PWM16_PHASE_CORRECT_MODE_8BIT, _PWM16_PRESCALER_16bit_8, _PWM16_NON_INVERTED, 255, 1);  // PWM Init
      PWM16bit_Start(_TIMER1);
      PWM16bit_Change_Duty(current_speed, _TIMER1_CH_B);

      Delay_ms(100);

      current_speed = 0; //initial value for current duty
    }

    void main() {
        MCU_Init();
        Lcd_Init();                        // Initialize LCD
        Lcd_Cmd(_LCD_CLEAR);               // Clear display
        Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off
        Lcd_Out(1, 1, txt1);
        Lcd_Out(2, 1, txt2);
        WordToStr(pwm_period, txt);
        Lcd_Out(2, 10, txt);

        DDB0_bit = 0;                      // Set PORTB pin 0 as input
        DDB1_bit = 0;                      // Set PORTB pin 1 as input
        DDA6_bit = 1;                      // Set PORTA pin 6 as output.
        DDD4_bit = 1;                      // Set PORTD pin 4 as output pin for the PWM (according to datasheet) OC1B, Output Compare Match B output: The PD4 pin can serve as an external output for theTimer/Counter1 Output Compare B
        MOTOR_DIR = CCW;

        while(1) {

          if (PINB0_bit)                  // button on PB0 pressed
           {
            Delay_ms(1);
            current_speed = current_speed + STEP;   // increment current_duty
            if (current_speed > pwm_period)       // if we increase current duty greater then possible pwm_period value
            {
              current_speed = 0;                // reset current duty value to zero
            }
            PWM16bit_Change_Duty(current_speed, _TIMER1_CH_B);

           }

          if (PINB1_bit)                  // button on PB1 pressed
           {
            Delay_ms(1);
            current_speed = current_speed - STEP;    // decrement current_duty
            if (current_speed > pwm_period)       // if we decrease current duty below zero value
            {
              current_speed = pwm_period;         // set current duty value to pwm_period
            }
            PWM16bit_Change_Duty(current_speed, _TIMER1_CH_B);
           }

           Delay_ms(20);

           WordToStr(current_speed, txt);
           Lcd_Out(1, 10, txt);

        }

    }